<?php
class DataCheck extends DBBase
{
    public static function checkStep3()
    {
        $errors = [];
        $startstation = Session::getStartStation();
        if (empty($startstation)) {
            $errors[] = ErrorList::getError('ERROR_STARTSTATION');
        }

        $endstation = Session::getEndStation();
        if (empty($endstation)) {
            $errors[] = ErrorList::getError('ERROR_ENDSTATION');
        }

        $startdatetime = Session::getStartDateTime();
        if (empty($startdatetime)) {
            $errors[] = ErrorList::getError('ERROR_STARTDATETIME');
        }

        $enddatetime = Session::getEndDateTime();
        if (empty($enddatetime)) {
            $errors[] = ErrorList::getError('ERROR_ENDDATETIME');
        }

        $car_group = Session::getCarGroup();
        if (empty($car_group)) {
            $errors[] = ErrorList::getError('ERROR_NO_GROUP');
        }

        if (!empty($errors)) {
            goError($errors);
        }
    } // checkStep3()

    public static function checkStep4()
    {
        $customerObj = unserialize($_SESSION['DATA']['customer']);
        $_SESSION['ERROR_LIST'] = [];
        self::checkStep3();

        if (!is_a($customerObj, 'Customer') or empty($customerObj)) {
            $errors[] = ErrorList::getError('ERROR_CUSTOMER_DATA');
            goError($errors);
        }

        $rate = unserialize($_SESSION['DATA']['rates']['primary'])[0];
        if (!is_a($rate, 'Rate') or empty($rate)) {
            $errors[] = ErrorList::getError('ERROR_RATE_MISSING');
            goError($errors);
        }

        //$pflichtAttribute = array( 'firstname' , 'lastname' , 'street' , 'zipcode' , 'city' , 'phone' );
        $pflichtAttribute = ['street', 'zipcode', 'city', 'phone'];
        $anrede = new Anrede($customerObj->anredeID);
        if (!$anrede->isCompany) {
            $pflichtAttribute = array_merge($pflichtAttribute, ['firstname', 'lastname']);
        }
        # Check reines Pflichtfeld
        foreach ($pflichtAttribute as $curAttr) {
            if (strlen($customerObj->$curAttr) == 0) {
                $_SESSION['ERROR_FLAGS'][$curAttr] = $curAttr;
            }
        }

        # Check eMail-Feld
        if (strlen($customerObj->email) == 0 or !filter_var($customerObj->email, FILTER_VALIDATE_EMAIL)) {
            $_SESSION['ERROR_FLAGS']['email'] = 'email';
        }
        if (!$customerObj->anrede) {
            $_SESSION['ERROR_FLAGS']['anrede'] = 'anrede';
        }
        $module = new Module();
        if (!$customerObj->payment) {
            $_SESSION['ERROR_FLAGS']['payment'] = 'payment';
        }
        if (!empty($_SESSION['ERROR_FLAGS']) and !array_key_exists('AGB', $_SESSION['ERROR_FLAGS'])) {
            $_SESSION['error_flag'] = 1;

            $_SESSION['ERROR_LIST'] = ErrorList::getError('ERROR_CUSTOMER_DATA');
            header('location: ' . SITEURL . 'summary&PHPSESSID=' . session_id());
            exit;
        } else {
            $_SESSION['error_flag'] = 0;
        }
    } // checkStep4

    public static function checkStep5()
    {
        self::checkStep4();

        $_SESSION['ERROR_LIST'] = [];
        $post = Session::getPost();

        if (!isset($post['checkbox_AGB']) || !isset($post['checkbox_DS'])) {
            if (!isset($post['checkbox_AGB'])) {
                $_SESSION['ERROR_FLAGS']['AGB'] = 'AGB';
                $_SESSION['ERROR_LIST'][] = ErrorList::getError('ERROR_ACCEPT_AGB');
            }
            if (!isset($post['checkbox_DS'])) {
                $_SESSION['ERROR_FLAGS']['DS'] = 'DS';
                $_SESSION['ERROR_LIST'][] = ErrorList::getError('ERROR_ACCEPT_DS');
            }
            header('location: ' . SITEURL . 'step5&PHPSESSID=' . session_id());
            exit;
        }
    } // checkStep5()
} // Check
