<?php

class Mail
{

    private $mail;

    public $is_bcc = true;

    public function __construct()
    {
        global $globalcfg;

        $SMAIL_SERVER = Params::getValue('SMAIL_SERVER');
        $SMAIL_SMTP_PORT = Params::getValue('SMAIL_SMTP_PORT');
        $SMAIL_SMTP_SEC = Params::getValue('SMAIL_SMTP_SEC');
        $SMAIL_SMTP_PW = Params::getValue('SMAIL_SMTP_PW');
        $SMAIL_SMTP_USER = Params::getValue('SMAIL_SMTP_USER');
        $SMAIL_STANDARDBCC = Params::getValue('SINET_MAIL_STANDARDBCC');
        $SMAIL_STANDARDCC = Params::getValue('SINET_MAIL_STANDARDCC');
        $SMAIL_MSG_CHARSET = Params::getValue('SMAIL_MSG_CHARSET');
        // $SMAIL_MSG_TYPE = Params::getValue('SMAIL_MSG_TYPE');
        $SMAIL_SENDER = Params::getValue('SINET_MAIL_FROM');
        $SMAIL_SENDERNAME = Params::getValue('SINET_MAIL_FROM_NAME');

        $station = Session::getStartStation();
        if (empty($station)) {
            $station = Station::getZentrale();
        }

        if ($SMAIL_SENDER == '') {
            $SMAIL_SENDER = $station->email;
        }

        if ($SMAIL_SENDERNAME == '') {
            $SMAIL_SENDERNAME = $station->name;
        }

        if (!empty($SMAIL_SERVER) && !empty($SMAIL_SMTP_PW) && !empty($SMAIL_SMTP_USER)) {
            $smtp_server = $SMAIL_SERVER;
            $smtp_password = $SMAIL_SMTP_PW;
            $smtp_user = $SMAIL_SMTP_USER;
            $smtp_port = (!empty($SMAIL_SMTP_PORT) && (intval($SMAIL_SMTP_PORT) > 0)) ? $SMAIL_SMTP_PORT : 25;
            $smtp_secure = (!empty($SMAIL_SMTP_SEC)) ? strtolower($SMAIL_SMTP_SEC) : '';
            $smtp_sender = $SMAIL_SENDER;
            $smtp_sendername = $SMAIL_SENDERNAME;
            $smtp_cc = ($SMAIL_STANDARDCC !== null) ? $SMAIL_STANDARDCC : '';
            $smtp_bcc = ($SMAIL_STANDARDBCC !== null) ? $SMAIL_STANDARDBCC : '';
        } else {
            $smtp_server = $globalcfg['email']['smtp'];
            $smtp_password = $globalcfg['email']['smtp_password'];
            $smtp_user = $globalcfg['email']['smtp_username'];
            $smtp_port = (!empty($globalcfg['email']['smtp_port']) && (intval($globalcfg['email']['smtp_port']) > 0)) ? $globalcfg['email']['smtp_port'] : 25;
            $smtp_secure = (!empty($globalcfg['email']['smtp_sec'])) ? strtolower($globalcfg['email']['smtp_sec']) : '';
            $smtp_sender = $SMAIL_SENDER;
            $smtp_sendername = $SMAIL_SENDERNAME;
            $smtp_cc = isset($globalcfg['email']['cc']) ? $globalcfg['email']['cc'] : '';
            $smtp_bcc = isset($globalcfg['email']['bcc']) ? $globalcfg['email']['bcc'] : '';
        }

        // AK 22.02.19 Aufgabe 1103673: Die Initialisierung des CX9Mailers überschreibt die Daten, daher muss diese davor stattfinden.
        $this->mail = new CX9Mailer();

        if (!empty($smtp_user) and !empty($smtp_password)) {
            $this->mail->SMTPAuth = true;
            $this->mail->Username = $smtp_user;
            $this->mail->Password = $smtp_password;
        } else {
            $this->mail->SMTPAuth = false;
            $this->mail->Username = '';
            $this->mail->Password = '';
        }

        $this->mail->CharSet = 'utf-8';

        $this->mail->IsSMTP(); // telling the class to use SMTP
        $this->mail->SMTPSecure = $smtp_secure;
        $this->mail->Host = $smtp_server; // SMTP server
        $this->mail->Port = $smtp_port;

        $this->mail->FromName = $smtp_sendername;
        $this->mail->From = $smtp_sender;

        if (trim($smtp_cc) != '') {
            $cc = explode(';', $smtp_cc);
            foreach ($cc as $address) {
                $address = trim($address);
                if ($address != '') {
                    $this->mail->AddCC($address);
                }
            }
        }

        if ($this->is_bcc && trim($smtp_bcc) != '') {
            $bcc = explode(';', $smtp_bcc);
            foreach ($bcc as $address) {
                $address = trim($address);
                if ($address != '') {
                    $this->mail->AddBCC($address);
                }
            }
        }

        if (Params::getValue('SMAIL_SENDERBCC') == 1) {
            $this->mail->AddBCC($this->mail->From);
        }

        if (Params::getValue('BINET_MAIL_BCC') == 1) {
            $this->mail->AddBCC($this->mail->From);
        }

        $this->mail->isHtml(true);
    }
    // __construct()
    public function Send($to, $subject, $body)
    {
        $this->mail->Subject = $subject;
        $this->mail->MsgHTML($body);

        $to = explode(';', $to);
        foreach ($to as $address) {
            $address = trim($address);
            if ($address != '') {
                $this->mail->AddAddress($address);
            }
        }

        return $this->mail->Send();
    }
    // Send()
    // AK 25.09.2015 Aufgabe 71678 : Sender soll Mailadresse vom Kunden sein, default ist $station->email
    public function setSender($sendermail)
    {
        if ($sendermail != '') {
            $this->mail->From = $sendermail;
        }
    }

    public function ErrorInfo()
    {
        return $this->mail->ErrorInfo;
    }

    public function addStringAttachment($string, $filename, $encoding = 'base64', $type = 'application/octet-stream')
    {
        $this->mail->addStringAttachment($string, $filename, $encoding, $type);
    }

    public function getLog()
    {
        $log = 'Absender: ' . $this->mail->From . "\n";
        $log .= 'Empfänger: ' . $this->mail->getAddresses() . "\n";
        $log .= 'CC: ' . $this->mail->getCCAddresses() . "\n";
        $log .= 'BCC: ' . $this->mail->getBCCAddresses() . "\n";
        $log .= 'Betreff: ' . $this->mail->Subject . "\n";
        $log .= 'Anhänge: ' . $this->mail->getAttachments() . "\n";

        return $log;
    }

    public function log2db($status, $reservationnumer = '0')
    {
        $reg = Registry::getInstance();
        $db = $reg->db;

        $sql = "SELECT MAX(PK)+1 AS PK FROM MAIL_MAILS";
        $db->prepare($sql);
        $result = $db->execute();

        $pk = 0;
        if ($result !== false) {
            $row = $db->fetch_assoc();
            $pk = isset($row['PK']) ? $row['PK'] : 0;
        }

        $sql = "INSERT INTO dbo.MAIL_MAILS
                ( PK ,
                  EmpfaengerAdresse ,
                  Subject ,
                  Body ,
                  gesendet ,
                  crentuser ,
                  Fehlermeldung ,
                  Erfassungsdatum ,
                  Letzterversuch ,
                  VerbleibendeVersuche ,
                  SenderAdresse ,
                  SenderName ,
                  EmpfaengerName ,
                  MailType ,
                  DocNumber
                )
        VALUES  ( :pk,
                  :receiver,
                  :subject,
                  :body,
                  GETDATE(),
                  'WEB',
                  :status,
                  NULL,
                  NULL,
                  NULL,
                  :sender,
                  :sendername,
                  :receivername,
                  'RES_BEST',
                  :reservationnumber
                )";
        $db->prepare($sql);

        $db->bindValue('pk', $pk);
        $db->bindValue('receiver', $this->mail->getAddresses());
        $db->bindValue('receivername', ''); // TODO Receiver Name
        $db->bindValue('sender', $this->mail->From);
        $db->bindValue('sendername', $this->mail->FromName);
        $db->bindValue('subject', $this->mail->Subject);
        $db->bindValue('body', $this->mail->Body);
        $db->bindValue('status', $status);
        $db->bindValue('reservationnumber', $reservationnumer);

        $db->execute();
    }
} // Class Mail
