<?php
class Session
{
    public static function getLang()
    {
        $lang = !empty($_SESSION['CONFIG']['LANG']) ? $_SESSION['CONFIG']['LANG'] : (defined('DEFAULT_LANG') && DEFAULT_LANG != '' ? DEFAULT_LANG : 'DE');
        return $lang;
    }

    public static function setLang($lang)
    {
        $_SESSION['CONFIG']['LANG'] = !empty($lang) ? $lang : self::getLang();
    }

    public static function getStartStation()
    {
        $station = unserialize($_SESSION['DATA']['startstation']);
        $station = is_a($station, 'Station') ? $station : null;

        return $station;
    } // getStartStation()

    public static function setStartStation($station)
    {
        if (is_a($station, 'Station')) {
            $_SESSION['DATA']['startstation'] = serialize($station);
        }
    } // setStartStation()

    public static function getEndStation()
    {
        $station = unserialize($_SESSION['DATA']['endstation']);
        $station = is_a($station, 'Station') ? $station : null;
        return $station;
    } // getEndStation()

    public static function setEndStation($station)
    {
        if (is_a($station, 'Station')) {
            $_SESSION['DATA']['endstation'] = serialize($station);
        }
    } // setEndStation()

    public static function getStationsList()
    {
        $stations = [];
        if (!empty($_SESSION['DATA']['stationslist']) && is_array(unserialize($_SESSION['DATA']['stationslist']))) {
            $stations = unserialize($_SESSION['DATA']['stationslist']);
        }
        return $stations;
    } // getStationsList()

    public static function setStationsList($stations)
    {

        if (is_a($stations, 'Station')) {
            $stations = array($stations->code => $stations);
        } elseif (is_array($stations)) {
            if (key_exists(0, $stations)) {
                $tmp = $stations;
                $stations = array();
                for ($i = 0; $i < count($tmp); $i++) {
                    $stations[$tmp[$i]->code] = $tmp[$i];
                }
            }
        } else {
            $stations = array();
        }

        $_SESSION['DATA']['stationslist'] = serialize($stations);
    } // setStationsList()

    public static function getCarGroup()
    {
        $group = unserialize($_SESSION['DATA']['car_group']);
        $group = is_a($group, 'CarsGroup') ? $group : null;
        return $group;
    } // getCarGroup()

    public static function setCarGroup($car_group_obj)
    {
        if (is_a($car_group_obj, 'CarsGroup')) {
            $_SESSION['DATA']['car_group'] = serialize($car_group_obj);
            $_SESSION['CARS']['GROUPS'][$car_group_obj->code] = serialize($car_group_obj);
        }
    } // setCarGroup()

    public static function delCarGroup()
    {
        $_SESSION['DATA']['car_group'] = null;
        unset($_SESSION['DATA']['car_group']);
    } // setCarGroup()

    public static function getPrimaryRate()
    {
        $error = false;

        $rates = unserialize($_SESSION['DATA']['rates']['primary']);
        if (!empty($rates) && is_array($rates)) {
            foreach ($rates as $rate) {
                if (!is_a($rate, 'Rate')) {
                    $error = true;
                    break;
                }
            }
        } else {
            $error = true;
        }
        $rates = $error ? null : $rates;
        return $rates;
    } // getPrimaryRate()

    public static function setPrimaryRates($rates)
    {
        try {
            if (!is_array($rates)) {
                throw new Exception('no Rtaes');
            }

            foreach ($rates as $rate) {
                if (!is_a($rate, 'Rate')) {
                    throw new Exception('no Rtaes');
                }
            }
            $_SESSION['DATA']['rates']['primary'] = serialize($rates);
        } catch (Exception $ex) {
            /* nothing */
        }
    } // setPrimaryRates()

    public static function getSubRates()
    {
        $error = false;
        $rates = [];
        if (!empty($_SESSION['DATA']['rates']['subrates']['options'])) {
            $session_rates = unserialize($_SESSION['DATA']['rates']['subrates']['options']);
            if (!empty($session_rates) && is_array($session_rates)) {
                foreach ($session_rates as $rate) {
                    if (is_a($rate, 'Rate')) {
                        $rates[] = $rate;
                    }
                }
            }
        }
        return $rates;
    } // getSubRates()

    public static function setSubRates($subrates)
    {
        try {
            if (!is_array($subrates)) {
                throw new Exception('no Rtaes');
            }

            foreach ($subrates as $rate) {
                if (!is_a($rate, 'Rate')) {
                    throw new Exception('no Rtaes');
                }
            }
            $_SESSION['DATA']['rates']['subrates']['options'] = serialize($subrates);
        } catch (Exception $ex) {
            /* nothing */
        }
    } // setSubRates()

    public static function getRentalHash()
    {
        $user = self::getCustomer();
        $obj_str = '';
        $obj_str .= serialize(self::getStartDateTime());
        $obj_str .= serialize(self::getEndDateTime());
        $obj_str .= serialize(self::getStartStation());
        $obj_str .= serialize(self::getCarGroup());
        $obj_str .= serialize(self::getDiscount());
        $obj_str .= serialize($_SESSION['ANGEBOTPK']);
        $obj_str .= !empty($user->id) ? $user->id : 0;
        $obj_str .= serialize(self::getAddKm());

        return md5($obj_str);
    } // getRentalHash()

    public static function getSessionRentalHash()
    {
        return !empty($_SESSION['CACHE']['RentalHash']) ? $_SESSION['CACHE']['RentalHash'] : '';
    } // getSessionRentalHash()

    public static function setSessionRentalHash()
    {
        $_SESSION['CACHE']['RentalHash'] = self::getRentalHash();
    } // setSessionRentalHash()

    public static function setCarGrpCalcHash()
    {
        $customer = self::getCustomer();
        $customerID = ((int) $customer->id > 0) ? (int) $customer->id > 0 : 0;
        $angebotPK = (isset($_SESSION['ANGEBOTPK'])) ? $_SESSION['ANGEBOTPK'] : 0;
        $hashParams = array(
            self::getStartStation()->code,
            self::getEndStation()->code,
            self::getStartDateTime() * 1000,
            self::getEndDateTime() * 1000,
            $customerID,
            $angebotPK,
        );
        $_SESSION['CACHE']['CARGRP_PRICES_HASH'] = self::createPricesHash($hashParams);
    }

    public static function getCarGrpCalcHash()
    {
        return (isset($_SESSION['CACHE']['CARGRP_PRICES_HASH'])) ? $_SESSION['CACHE']['CARGRP_PRICES_HASH'] : false;
    }

    public static function createPricesHash($hashParams)
    {
        return md5(implode(',', $hashParams));
    }

    public static function getStations()
    {
        $stations = !empty($_SESSION['DATA']['stations']) ? unserialize($_SESSION['DATA']['stations']) : array();
        if (empty($stations[0]) || !is_a($stations[0], 'Stations')) {
            $stations = array();
        }
        return $stations;
    } // getStations()

    public static function setStations($stations)
    {
        $_SESSION['DATA']['stations'] = serialize($stations);
    } // setStations()

    public static function getStation($code)
    {
        $station = null;
        $stations = self::getStations();
        foreach ($stations as $st) {
            if ($st->code == $code) {
                $station = $st;
                break;
            }
        }
        return $station;
    } // getStation()

    public static function setStation($station)
    {
        $found = false;
        $stations = self::getStations();
        foreach ($stations as &$st) {
            if ($st->code == $station->code) {
                $st = $station;
                $found = true;
                break;
            }
        }
        unset($st);

        if (!$found) {
            $stations[] = $station;
        }

        self::setStations($stations);
    } // setStation()

    public static function getPost()
    {
        return !empty($_SESSION['POST']) ? $_SESSION['POST'] : null;
    } // getPost()

/**
 *
 * @param type $discount Array
 * $discount[code]
 * $discount[persent]
 * $discount[summe]
 * $discount[text]
 * $discount[disable]
 *
 */
    public static function setDiscount($discount)
    {
        $_SESSION['DATA']['discount'] = $discount;
    } // setDiscount()

    public static function getDiscount()
    {
        return $_SESSION['DATA']['discount'];
    } // getDiscount()

    public static function delDiscount()
    {
        $_SESSION['DATA']['discount'] = null;
        unset($_SESSION['DATA']['discount']);
    } // delDiscount()

    public static function delAllRates()
    {
        $_SESSION['DATA']['rates'] = null;
        unset($_SESSION['DATA']['rates']);
    } // delAllRates()

    public static function getCustomer()
    {
        $customer = null;
        if (!empty($_SESSION['DATA']['customer'])) {
            $customer = unserialize($_SESSION['DATA']['customer']);
        }
        return is_a($customer, 'Customer') ? $customer : null;
    } // getCustomer()

    public static function setCustomer($customer)
    {
        if (is_a($customer, 'Customer')) {
            $_SESSION['DATA']['customer'] = serialize($customer);
        }
    } // setCustomer()

    public static function delCustomer()
    {
        $_SESSION['DATA']['customer'] = null;
        unset($_SESSION['DATA']['customer']);
    } // setCustomer()

    public static function getStartDateTime()
    {
        return isset($_SESSION['DATA']['startdatetime']) ? $_SESSION['DATA']['startdatetime'] : null;
    } // getStartDateTime()

    public static function setStartDateTime($timestamp)
    {
        $_SESSION['DATA']['startdatetime'] = (float) $timestamp;
    } // setStartDateTime()

    public static function getEndDateTime()
    {
        return isset($_SESSION['DATA']['enddatetime']) ? $_SESSION['DATA']['enddatetime'] : null;
    } // getEndDateTime()

    public static function setEndDateTime($timestamp)
    {
        $_SESSION['DATA']['enddatetime'] = (float) $timestamp;
    } // setEndDateTime()

    public static function getAddKm()
    {
        return !empty($_SESSION['DATA']['addkm']) ? intval($_SESSION['DATA']['addkm']) : 0;
    } // getAddKm()

    public static function setAddKm($km)
    {
        $_SESSION['DATA']['addkm'] = (int) $km;
    }

    public static function delAddKm()
    {
        $_SESSION['DATA']['addkm'] = null;
        unset($_SESSION['DATA']['addkm']);
    }

    public static function getCurrency()
    {
        $currency = null;

        if (!empty($_SESSION['CONFIG']['currency_obj'])) {
            $currency = unserialize($_SESSION['CONFIG']['currency_obj']);
        }
        return is_a($currency, 'Currency') ? $currency : null;
    }

    public static function setCurrency(Currency $currency_obj)
    {
        if (is_a($currency_obj, 'Currency')) {
            $_SESSION['CONFIG']['currency_obj'] = serialize($currency_obj);
        }
    }

    public static function getStationsDep()
    {
        return unserialize($_SESSION['CACHE']['STATIONS_DEP']);
    }
    public static function setStationsDep($stations_dep)
    {
        $_SESSION['CACHE']['STATIONS_DEP'] = serialize($stations_dep);
    }
    public static function getStationsArr()
    {
        return unserialize($_SESSION['CACHE']['STATIONS_ARR']);
    }
    public static function setStationsArr($stations_arr)
    {
        $_SESSION['CACHE']['STATIONS_ARR'] = serialize($stations_arr);
    }
} // Class Session
