<?php
class Station extends DBBase
{
    public $code = '';
    public $name = '';
    public $country = '';
    public $stat = '';
    public $street = '';
    public $city = '';
    public $zipcode = '';
    public $phone = '';
    public $fax = '';
    public $email = '';
    public $url = '';
    public $onlyci = 0;
    public $isOneWay = 0;
    public $position = null;

    public $shedules = array();
    public $min_vorlauf = 0;

    public function __construct($code = '')
    {
        parent::__construct();
        if (!empty($code)) {
            $result = $this->getDatabase()->proc_exec('INET_GET_STATIONS_INFO', array('station' => $code));

            if ($result !== false) {
                $row = $this->getDatabase()->fetch_assoc();
                if (!empty($row)) {
                    $this->code = $code;
                    $this->station = $code;
                    $this->name = !empty($row['INET_NAME']) ? $row['INET_NAME'] : $row['name'];
                    $this->stat = !empty($row['bundeslandkuerzel']) ? $row['bundeslandkuerzel'] : (!empty($row['Bundeslandkuerzel']) ? $row['Bundeslandkuerzel'] : '');
                    $this->street = $row['strasse'];
                    $this->city = $row['ort'];
                    $this->zipcode = $row['plz'];
                    $this->phone = !empty($row['Telefon']) ? $row['Telefon'] : '';
                    $this->fax = !empty($row['Fax']) ? $row['Fax'] : '';
                    $this->email = !empty($row['EMail']) ? $row['EMail'] : '';

                    $sql = "SELECT INET_MIN_VORLAUF,url,land, ISNULL(isOneWay,0) AS isOneWay FROM dbo.stammt WHERE station = :code";
                    $this->db->prepare($sql);
                    $this->db->bindValue(':code', $code);
                    $this->db->execute();
                    $row2 = $this->db->fetch_assoc();
                    $this->url = is_null($row2['url']) ? '' : $row2['url'];
                    $this->min_vorlauf = is_null($row2['INET_MIN_VORLAUF']) ? 0 : intval($row2['INET_MIN_VORLAUF']);
                    $this->onlyCI = 0;
                    $this->isOneWay = $row2['isOneWay'];
                    if (!empty($row2['land'])) {
                        $sql = "SELECT land FROM LAENDERKENNZEICHEN
                                WHERE Kennzeichen = :land";
                        $this->db->prepare($sql);
                        $this->db->bindValue(':land', $row2['land']);
                        $result = $this->db->execute();
                        if ($result !== false) {
                            $row3 = $this->getDatabase()->fetch_assoc();
                            if (!empty($row3['land'])) {
                                $this->country = $row3['land'];
                            }
                        }
                    }

                    $position = new Position();
                    if (!empty($row2['INET_GOOGLE_MAPS_LINK'])) {
                        $tmp_pos = explode(',', $row2['INET_GOOGLE_MAPS_LINK']);
                        if (!empty($tmp_pos[0]) && !empty($tmp_pos[1])) {
                            $position->lat = trim($tmp_pos[0]);
                            $position->long = trim($tmp_pos[1]);
                        }
                    }
                    $this->position = $position;

                    $schedule = new StationSchedule();
                    $this->shedules = $schedule->getSchedule($this->code);
                    $this->shedulesPrepare();
                }
            }
        }
    } // __construct

    public static function getZentrale()
    {
        $zentrale_id = '';
        $reg = Registry::getInstance();
        $db = $reg->db;
        $db->query('SELECT Zentrale FROM MAND_GLOBAL LEFT JOIN dbo.stammt ON MAND_GLOBAL.Zentrale = stammt.station WHERE Internetstation = 1');
        $row = $db->fetch_assoc();
        if (!empty($row['Zentrale'])) {
            $zentrale_id = $row['Zentrale'];
        }

        return new self($zentrale_id);
    }

    public function isOpen($timestamp)
    {
        $dbStation = new DBStation();
        $this->open = $dbStation->is_open_db($this->code, $timestamp);

        return $this->open;
    } //isOpen()

    public function getFirstOpenTime($date = '')
    {
        $vorlauf_step = 30;
        $date = ($date == '' ? time() : strtotime($date));
        // calculate min time (current time + forerrun)
        $min_date = strtotime(date('d.m.Y H:i') . ' +' . $this->min_vorlauf . ' minutes');

        if ($date < $min_date) {
            $date = $min_date;
        }
        // round time up to half hour or full hour
        $date = strtotime(date('d.m.Y H', $date) . ':' . (intval(date('i', $date)) <= 30 ? '30' : '00 +1 hours'));
        $stop_date = strtotime(date('d.m.Y H:i', $date) . ' + 1 week');
        while (!$this->isOpen($date) && $date < $stop_date) {
            $date = strtotime(date('d.m.Y H:i', $date) . ' + 30 minutes');
        }

        return $date < $stop_date ? $date : false;
    } //getFirstOpenTime()

    private function shedulesPrepare()
    {
        $week_days = array('mo', 'di', 'mi', 'do', 'fr', 'sa', 'so');
        $week_tmp_arr = array();

        for ($i = 0; $i < count($this->shedules); $i++) {
            foreach ($this->shedules[$i] as $key => &$val) {
                if ($key == 'day') {
                    $val = strtolower($val);
                    $week_tmp_arr[] = $val;
                }
            }
        }
    } // shedulesPrepare()

    public static function updatePosition()
    {
        $dbStation = new DBStation();
        if (!$dbStation->isCheckSumm()) {
            foreach ($dbStation->getAllStations() as $station) {
                $response = file_get_contents('http://maps.googleapis.com/maps/api/geocode/json?address=' .
                    urlencode($station->strasse) . '+' .
                    urlencode($station->plz) . '+' .
                    urlencode($station->ort) . '+' .
                    (empty($station->land) ? $station->land : 'DE') . '&sensor=false');
                $response = json_decode($response);
                if ($response->status == 'OK') {
                    $lat = $response->results[0]->geometry->location->lat;
                    $long = $response->results[0]->geometry->location->lng;
                    $dbStation->positionUpdate($station->station, $lat, $long);
                }
            }

            $dbStation->setCheckSum();
        }
    } // updatePosition();

    public function getOpenTime($start_end = '')
    {

        $date = new DateTime();
        $ignor_is_open = false;
        $station_shedules = [];
        $raw = [];
        foreach ($this->shedules as $val) {
            switch ($val->day) {
                case 'so':
                    $station_shedules[0] = $val;
                    break;
                case 'mo':
                    $station_shedules[1] = $val;
                    break;
                case 'di':
                    $station_shedules[2] = $val;
                    break;
                case 'mi':
                    $station_shedules[3] = $val;
                    break;
                case 'do':
                    $station_shedules[4] = $val;
                    break;
                case 'fr':
                    $station_shedules[5] = $val;
                    break;
                case 'sa':
                    $station_shedules[6] = $val;
                    break;
            }
        }

        $tmp = $station_shedules;
        $min_date_time = $date->add(new DateInterval('PT' . $this->min_vorlauf . 'M'));

        if ($start_end == 'start' && Params::getValue('CONFIG_EVERYTIME_DEPARTURE') == 1) {
            foreach ($station_shedules as &$val) {
                $val->mbtime = '00:00';
                $val->metime = '00:00';
                if ($val->stationClosed) {
                    $val->btime = '08:00';
                    $val->etime = '18:00';
                    $val->stationClosed = false;
                }
                $val->btime = $val->btime > '08:00' ? '08:00' : $val->btime;
                $val->etime = $val->etime < '18:00' ? '18:00' : $val->etime;
            }
            unset($val);
            $min_date_time = new DateTime();
            $ignor_is_open = true;
        }

        if ($start_end == 'end' && Params::getValue('CONFIG_EVERYTIME_RETURN') == 1) {
            foreach ($station_shedules as &$val) {
                $val->btime = '00:00';
                $val->etime = '23:59';
                $val->mbtime = '00:00';
                $val->metime = '00:00';
                $val->stationClosed = false;
            }
            unset($val);
            $min_date_time = new DateTime();
            $ignor_is_open = true;
        }
        $time_step = Params::getValue('INET_OPENINGTIMES_STEPS');
        $time_step = !empty($time_step) ? (int) $time_step : 30;
        if ($min_date_time->getTimestamp() % ($time_step * 60) != 0) {
            $min_date_time->add(new DateInterval('PT' . (($time_step * 60) - $min_date_time->getTimestamp() % ($time_step * 60)) . 'S'));
        }

        if (!$ignor_is_open) {
            while (!$this->isOpen($min_date_time->getTimestamp())) {
                $min_date_time->add(new DateInterval('PT' . $time_step . 'M'));
            }
        }

        $min_date_shedules = $station_shedules[$min_date_time->format('N') - 1];
        $station_shedules[] = $min_date_shedules;

        $shedule = array();
        for ($i = 0; $i < count($station_shedules); $i++) {
            $mittag = $station_shedules[$i]->mbtime != $station_shedules[$i]->metime;
            if ($station_shedules[$i]->btime == $station_shedules[$i]->etime) {
                $shedule[$i] = array();
                continue;
            }
            if ($i != count($station_shedules) - 1) {
                $tmp_time = new DateTime(date('Y-m-d ' . $station_shedules[$i]->btime));
            } else {
                $tmp_time = new DateTime();
                $tmp_time->setTimestamp($min_date_time->getTimestamp());
            }

            $curent_hour = (int) $tmp_time->format('H');
            $time_stop = new DateTime($tmp_time->format('Y-m-d') . ' ' . $station_shedules[$i]->etime);

            $minutes = array();
            while ($tmp_time <= $time_stop) {
                if ($mittag && $tmp_time->format('H:i') > $station_shedules[$i]->mbtime && $tmp_time->format('H:i') < $station_shedules[$i]->metime) {
                    $tmp_time->add(new DateInterval('PT' . $time_step . 'M'));
                    continue;
                }
                if ($curent_hour != $tmp_time->format('H')) {
                    $shedule[$i][] = array('hour' => $curent_hour, 'minutes' => $minutes);
                    $curent_hour = (int) $tmp_time->format('H');
                    $minutes = array();
                }
                // probe
                $minutes[] = $tmp_time->format('i');
                $tmp_time->add(new DateInterval('PT' . $time_step . 'M'));
            }
            if (!empty($minutes)) {
                $shedule[$i][] = array('hour' => $curent_hour, 'minutes' => $minutes);
            }
        }

        $min_opening = $shedule[count($shedule) - 1];
        unset($shedule[count($shedule) - 1]);

        if (!empty($station_shedules[7])) {
            unset($station_shedules[7]);
        }
        $return = array('MIN_DATE' => strtotime($min_date_time->format('Y-m-d')), 'MIN_TIME' => $min_date_time->format('H:i'), 'MIN_OPENINGS' => $min_opening, 'SHEDULE' => $shedule, 'RAW' => $station_shedules);
        return $return;
    } // getOpenTime()

    public static function getOpenTimeEveryTime($departure_return = '')
    {
        $time_step = Params::getValue('INET_OPENINGTIMES_STEPS');
        $time_step = !empty($time_step) ? (int) $time_step : 30;
        $min_date_time = new DateTime();
        $min_date_time->add(new DateInterval('PT1H'));
        $min_hour = $departure_return == 'departure' ? 8 : 0;
        $max_hour = $departure_return == 'departure' ? 18 : 23;
        $raw = [];
        for ($i = 0; $i < 7; $i++) {
            $raw[$i]['btime'] = ($min_hour > 9 ? $min_hour : '0' . $min_hour) . ':00';
            $raw[$i]['etime'] = ($max_hour > 9 ? $max_hour : '0' . $max_hour) . ':00';
            $raw[$i]['mbtime'] = '00:00';
            $raw[$i]['metime'] = '00:00';
            $raw[$i]['stationClosed'] = false;
        }
        if ($min_date_time->format('H:i') > '18:00') {
            $min_date_time->add(new DateInterval('P1D'));
            $min_date_time->setTime(8, 0);
        }
        $min_opening = array();
        for ($hour = (int) $min_date_time->format('H'); $hour <= $max_hour; $hour++) {
            $minutes = array();
            for ($minut = 0; $minut < 60; $minut += $time_step) {
                $time = ($hour > 9 ? $hour : '0' . $hour) . ':' . ($minut > 9 ? $minut : '0' . $minut);
                if ($time >= $min_date_time->format('H:i')) {
                    $minutes[] = DateTime::createFromFormat('H:i', $time)->format('i');
                }
                if ($departure_return == 'departure' && $hour == 18) {
                    break;
                }
            }
            if (!empty($minutes)) {
                $min_opening[] = array('hour' => $hour, 'minutes' => $minutes);
            }
        }

        $shedule = array();
        for ($i = 0; $i < 7; $i++) {
            for ($hour = $min_hour; $hour <= $max_hour; $hour++) {
                $minutes = array();
                for ($minut = 0; $minut < 60; $minut += $time_step) {
                    $minutes[] = $minut > 9 ? $minut : '0' . $minut;
                    if ($departure_return == 'departure' && $hour == 18) {
                        break;
                    }
                }
                $shedule[$i][] = array('hour' => $hour, 'minutes' => $minutes);
            }
        }

        $return = array('MIN_DATE' => strtotime($min_date_time->format('Y-m-d')), 'MIN_TIME' => $min_date_time->format('H:i'), 'MIN_OPENINGS' => $min_opening, 'SHEDULE' => $shedule, 'RAW' => $raw);
        return $return;
    } // getOpenTimeEveryTime()

    public static function getAll()
    {
        $registry = Registry::getInstance();
        $db = $registry->db;
        $stations = [];
        $db->query('SELECT station FROM dbo.stammt WHERE Internetstation = 1');
        $rows = $db->fetch_all();
        foreach ($rows as $row) {
            $station = new self($row['station']);

            $stations[] = $station;
        }

        return $stations;
    } // getAll()

    public static function getStationListFromCache($direction)
    {
        if (empty($_SESSION['CACHE']['STATIONS_DEP'])) {
            $_SESSION['CACHE']['STATIONS_DEP'] = serialize(self::getAll());
        }
        if ($direction == 'departure') {
            $stations = SESSION::getStationsDep();
        } else {
            if (empty($_SESSION['CACHE']['STATIONS_ARR'])) {
                $_SESSION['CACHE']['STATIONS_ARR'] = $_SESSION['CACHE']['STATIONS_DEP'];
            }
            $stations = SESSION::getStationsArr();
        }

        $dateTimeFormat = new DateTimeFormat();
        if (!empty($stations[0]->shedules)) {
            foreach ($stations[0]->shedules as &$val) {
                $val->btime = $dateTimeFormat->getTime($val->btime);
                $val->etime = $dateTimeFormat->getTime($val->etime);
                $val->mbtime = $dateTimeFormat->getTime($val->mbtime);
                $val->metime = $dateTimeFormat->getTime($val->metime);
            }
            unset($val);
        }
        return $stations;
    } // getStationList()

    public static function hasStationCode($station)
    {
        if (empty($station)) {
            $station = new \stdClass();
            $station->code = null;
        }
        return $station;
    }
    public function getStationsBlock()
    {
        $dbTranslate = new DBTranslate();
        $translate = array(
            'RENTAL' => $dbTranslate->loadTranslations('RENTAL', $this->registry->lang),
            'Wochentage' => $dbTranslate->loadTranslations('Wochentage', $this->registry->lang),
            'Wochentage_kurz' => $dbTranslate->loadTranslations('Wochentage_kurz', $this->registry->lang),
        );
        $regionsInstanz = new Region();
        $region = $regionsInstanz->getRegion();
        $dbStations = new DBStation();
        $stations_dep = array();
        $stations_arr = array();
        foreach ($dbStations->getAllStations(0, $region) as $dbStation) {
            $station = new self();

            $station->code = $dbStation->station;
            $station->name = (!empty($dbStation->INET_NAME)) ? $dbStation->INET_NAME : $dbStation->name;
            $station->country = $dbStation->land;
            $station->stat = $dbStation->bundeslandkuerzel;
            $station->street = $dbStation->strasse;
            $station->city = $dbStation->ort;
            $station->zipcode = $dbStation->plz;
            $station->phone = $dbStation->telefon;
            $station->fax = $dbStation->fax;
            $station->email = $dbStation->email;
            $station->url = $dbStation->url;
            $station->min_vorlauf = $dbStation->forerun;
            $station->onlyci = $dbStation->onlyCI;
            $station->isOneWay = empty($dbStation->isOneWay) ? false : $dbStation->isOneWay;
            $station->shedules = $dbStations->getStationSchedule($station->code);
            $week_days = array('mo', 'di', 'mi', 'do', 'fr', 'sa', 'so');
            $week_tmp_arr = array();

            for ($i = 0; $i < count($station->shedules); $i++) {
                foreach ($station->shedules[$i] as $key => &$val) {
                    if ($key != 'day' && $key != 'day_lang' && isset($val) && strlen($val) == 4) {
                        $val = substr($val, 0, 2) . ":" . substr($val, 2, 4);
                    } else {
                        if ($key != 'day_lang') {
                            $val = strtolower($val);
                            $week_tmp_arr[] = $val;
                        }
                    }
                }
            }

            $stations_arr[] = $station;

            if ($station->onlyci == 0) {
                $stations_dep[] = $station;
            }
        }
        SESSION::setStationsDep($stations_dep);
        SESSION::setStationsArr($stations_arr);
        if ((AJAX_DEBUG || @$_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest') && !empty($_GET['stations_typ'])) {
            $template = $this->template;
            $template->assign('STATIONS_DEP', $stations_dep);
            $template->assign('STATIONS_ARR', $stations_arr);
            $template->assign('TRANSLATE', $translate);

            $out = $template->fetch('station_list.html');

            echo $out;
        }
    } // getstationsBlock()
}
