<?php
class DBRate extends DBBase
{

    public function getStdRate($datetime_dep, $datetime_arr, $stationcode_dep, $stationcode_arr, $group_code, $lauf)
    {
        if (!empty($_SESSION['DATA']['customer'])) {
            $customer = unserialize($_SESSION['DATA']['customer']);
            if (!is_a($customer, 'Customer')) {
                $customer = new Customer();
            }
        } else {
            $customer = new Customer();
        }

        $globParams = new Params();
        $fzgArt = $this->handlePreislisteParams($group_code, $customer->id);
        $pricelist = $this->getPreisListe($_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params']);
        $vonRate = $this->getCustomerStandardRate($customer->id, $group_code);

        $params = array("Kunde" => $customer->id,
            "From" => $datetime_dep,
            "bis" => $datetime_arr,
            "Station" => $stationcode_dep,
            "StationAn" => $stationcode_arr,
            "Gruppe" => $group_code,
            "Inet" => 1,
            "Zusatztarife" => 0,
            "mitZusatzEinheit" => 0,
            "Kilometer" => 0,
            "Preisliste" => $pricelist,
            "Lauf" => 0,
            "alt" => 0,
            "VonRate" => $vonRate,
            "Gutscheincode" => '',
            "ARabatt" => 0,
            "FestRabatt" => 0,
            'fzgArt' => $fzgArt,
        );

        $result = $this->getDatabase()->proc_exec('RATES_GET_REL_RATES', $params);

        $pkrechnung = '';
        $first = true;
        $basic_price = 0;
        $total_price = 0;
        $tmp_rate = null;

        if ($result !== false) {
            while ($db_rate = $this->getDatabase()->fetch_obj()) {
                if ($first && $db_rate->MietRate == 'J') {
                    $tmp_rate = $db_rate;
                    $first = false;
                }
                if ($db_rate->PKRECHNUNG == $tmp_rate->PKRECHNUNG) {
                    $total_price += $db_rate->gesamtpreisbrutto;
                }
            }

            if (isset($tmp_rate)) {
                $tmp_rate->gesamtpreisbrutto = $total_price;
                $tmp_rate->days = $this->getDays($tmp_rate->PKRECHNUNG);
            }
        }

        return $tmp_rate;
    }

    public function getCustomerStandardRate($customerId, $groupCode)
    {
        if ((int) $customerId == 0 or strlen(trim($groupCode)) == 0) {
            return '';
        }

        $sql = 'SELECT TOP 1 Rate FROM KUNDEN_STANDARDRATEN WHERE Kundennummer = :customerId AND KFZGruppe = :groupCode';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':customerId', $customerId);
        $this->getDatabase()->bindValue(':groupCode', $groupCode);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $resArray = $this->getDatabase()->fetch_assoc();
            return (strlen(trim($resArray['Rate'])) > 0) ? $resArray['Rate'] : '';
        } else {
            return '';
        }
    }

    public function getPreisListe($array_input = array())
    {
        $actual_awdno = "noexistence";
        if (isset($_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params'][':AWDNo'])) {
            $actual_awdno = $_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params'][':AWDNo'];
        }

        $default_input = array(':vertragsnummer' => -1, ':kdno' => -1, ':akdno' => -1, ':param_preis_liste' => 'SINET_PREISLISTE', ':AWDNo' => '', ':fzgArt' => '');
        $real_db_fn_input = array_merge($default_input, $array_input);
        //print_r($real_db_fn_input);die;
        $list_result = 'OHNE';
        $sql_input_fields = implode(",", array_keys($real_db_fn_input));
        $sql = "SELECT dbo.ORM_GET_MV_PREISLISTE(" . $sql_input_fields . ") AS PreisList"; //echo $sql; die;
        try {
            $this->getDatabase()->prepare($sql);
            foreach ($real_db_fn_input as $key => $value) {
                $this->getDatabase()->bindValue($key, $value);
            }
            $result = $this->getDatabase()->execute();
            if ($result !== false) {
                $row = $this->getDatabase()->fetch_obj();
                $list_result = $row->PreisList;
            }
        } catch (Exception $e) {
            $list_result = 'OHNE';
        }

        return $list_result;
    }

    private function handlePreislisteParams($car_group, $customerId = -1)
    {
        $DBCarGroup = new DBCarGroup();
        $carGroupData = $DBCarGroup->getGroupByCode($car_group);
        $_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params'][':fzgArt'] = (isset($carGroupData->kfzart) ? $carGroupData->kfzart : '');

        $_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params'][':kdno'] = $customerId;

        return (isset($carGroupData->kfzart)) ? $carGroupData->kfzart : '';
    }

    public function getRate($date_dep, $date_arr, $start_station, $end_station, $car_group, $km = 0, $gutscheincode = '', $is_zusatz_tarif = 0, $summ_or_percent_rabat = 0, $fix_rabat = 0, $main_rate = null)
    {
        if (!empty($_SESSION['DATA']['customer'])) {
            $customer = unserialize($_SESSION['DATA']['customer']);
            if (!is_a($customer, 'Customer')) {
                $customer = new Customer();
            }
        } else {
            $customer = new Customer();
        }

        $globParams = new Params();
        $fzgArt = $this->handlePreislisteParams($car_group, $customer->id);
        $pricelist = $this->getPreisListe($_SESSION['DATA']['DB']['FUNCTION']['ORM_GET_MV_PREISLISTE']['params']);
        $vonRate = $this->getCustomerStandardRate($customer->id, $car_group);

        $rates = array();
        $params = array(
            'Kunde' => $customer->id,
            'From' => $date_dep,
            'bis' => $date_arr,
            'Station' => $start_station,
            'StationAn' => $end_station,
            'Gruppe' => $car_group,
            'Inet' => 1,
            'Zusatztarife' => $is_zusatz_tarif,
            'Kilometer' => $km,
            'Lauf' => 0,
            'Preisliste' => $pricelist,
            'alt' => 0,
            'VonRate' => (!is_null($main_rate) ? $main_rate->rate : $vonRate),
            'Gutscheincode' => $gutscheincode,
            'ARabatt' => $summ_or_percent_rabat,
            'FestRabatt' => $fix_rabat,
            'fzgArt' => $fzgArt,
        );
        $result = $this->getDatabase()->proc_exec('RATES_GET_REL_RATES', $params);

        if ($result !== false) {
            $db_rates_array = $this->getDatabase()->fetch_all();
            $categories = $this->getCategories();

            foreach ($db_rates_array as $db_rate) {
                $sql = "SELECT rrs.*, rr.INET_MAX_WERT, rr.INET_MIN_WERT , rr.PK_Kategorie_AUSW AS category FROM dbo.RECH_RATEN_STAFFELUNG rrs, dbo.RECH_RATEN rr WHERE rrs.rate = rr.rate AND rrs.rate = :rate AND (Gruppe = :gruppe OR Gruppe = '*')";
                $this->getDatabase()->prepare($sql);
                $this->getDatabase()->bindValue(':gruppe', $car_group);
                $rate = (object) $db_rate;
                $rate->min_count = 0;
                $rate->max_count = 10;
                $this->getDatabase()->bindParam(':rate', $rate->Rate);
                $this->getDatabase()->execute();
                $rows = $this->getDatabase()->fetch_all();
                $is_gruppe = false;
                foreach ($rows as $row) {
                    if (empty($row['PK'])) {
                        continue;
                    }

                    if ($is_gruppe && $row['Gruppe'] == '*') {
                        continue;
                    }

                    $rate->min_count = $rate->min_count > intval($row['INET_MIN_WERT']) ? intval($row['INET_MIN_WERT']) : $rate->min_count;
                    // AK 25.02.2016 - kleineren Wert von $row[Bis] und 10 nehmen
                    $rate->max_count = $rate->max_count < min(intval($row['Bis']), 10) ? min(intval($row['Bis']), 10) : $rate->max_count;

                    if (!$is_gruppe && $row['Gruppe'] == $car_group) {
                        $is_gruppe = true;
                        $rate->min_count = intval($row['Von']);
                        // AK 25.02.2016 - kleineren Wert von $row[Bis] und 10 nehmen
                        $rate->max_count = min(intval($row['Bis']), 10);
                    };

                    $rate->category = ((int) $row['category'] > 0) ? (int) $row['category'] : 0;
                    $rate->category_lang = ((int) $row['category'] > 0) ? $categories[$rate->category]['Kategorie'] : false;
                }

                $rate->days = $this->getDays($rate->PKRECHNUNG);

                if (!isset($rates[0]) || $rate->PKRECHNUNG == $rates[0]->PKRECHNUNG || $is_zusatz_tarif == 1) {
                    $rates[] = $rate;
                }
            }
        }

        return $rates;
    }

    public function getZusRate($date_dep = '', $date_arr = '', $start_station = '', $end_station = '', $car_group = '', $km = 0, $gutscheincode = '', $main_rate = null)
    {
        $rates = $this->getRate($date_dep, $date_arr, $start_station, $end_station, $car_group, $km, $gutscheincode, 1, 0, 0, $main_rate);

        if (!is_null($main_rate)) {
            foreach ($rates as &$rate) {
                $rate->autobuch = $this->is_autobuch($main_rate, $rate, $car_group, $date_dep, $date_arr);
            }
            unset($rate);
        }
        return $rates;
    }

    public function saveRate($array_rates, $res_num)
    {
        $sql = "UPDATE RECH_POS SET PKRECHNUNG = :res_num, Anzahl = :anzahl WHERE PK = :pk";

        $this->getDatabase()->prepare($sql);

        $this->getDatabase()->bindParam(':res_num', $res_num);
        $this->getDatabase()->bindParam(':anzahl', $anzahl);
        $this->getDatabase()->bindParam(':pk', $pk);

        foreach ($array_rates as $rate) {
            $anzahl = $rate['anzahl'];
            $pk = $rate['pk'];

            $this->getDatabase()->execute();
        }
    }

    public function isUnitTemporal($rate)
    {
        $temporal = true;

        $sql = "SELECT * FROM RECH_EINHEITEN WHERE Einheit = :unit";

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindParam(':unit', $rate->unit);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $row = $this->getDatabase()->fetch_obj();
            $temporal = !empty($row) && $row->zeitlich == 'J';
        }

        return $temporal;
    }

    public function isUnitStuk($rate)
    {
        $stuk = false;

        $sql = "SELECT * FROM RECH_EINHEITEN WHERE Einheit = :unit";

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':unit', $rate->unit);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $row = $this->getDatabase()->fetch_assoc();
            $stuk = $row['Einheit'] == 'Stück';
        }

        return $stuk;
    }

    public function isRabat($group, $rate, $discount_code)
    {
        $isRabat = false;
        $today = date("d.m.Y H:i");

        $sql = "SELECT nichtRabattfaehig FROM dbo.RECH_RATEN WHERE (Grp = '*' OR Grp = :group) AND Rate = :rate";

        $this->getDatabase()->prepare($sql);

        $this->getDatabase()->bindParam(':group', $group);
        $this->getDatabase()->bindParam(':rate', $rate->rate);

        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $row = $this->getDatabase()->fetch_obj();
            $isRabat = !isset($row->nichtRabattfaehig) || $row->nichtRabattfaehig != 1; //== 1 => non-discountable
        }

        if ($isRabat) {
            $sql = "SELECT *
					FROM INET_RABATT
					WHERE gutscheincode = :discount_code AND verwendet = 0 AND " . $today . " >= DATEADD(day, ISNULL(vorlauf, 0), von) AND " .
                $today . ">= DATEADD(day, ISNULL(vorlauf, 0), " . $today . ") AND
					ISNULL(verwendbar_bis, " . $today . ") >= " . $today;

            $this->getDatabase()->prepare($sql);
            $this->getDatabase()->bindParam(':discount_code', $discount_code);
            $result = $this->getDatabase()->execute();
            if ($result !== false) {
                $row = $this->getDatabase()->fetch_obj();
            } else {
                $error = 'Wrong code';
            }
        } else {
            $error = 'Wrong code';
        }
    }

    public function is_autobuch($rate, $subrate, $group, $date_dep, $date_arr)
    {
        $sql = "SELECT  [dbo].[INET_GET_ZUSRATE_OBLIG](:rate,
												:subrate,
												:group ,
												:date_dep,
												:date_arr) as autobuch";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':rate', $rate->rate);
        $this->getDatabase()->bindValue(':subrate', $subrate->Rate);
        $this->getDatabase()->bindValue(':group', $group);
        $this->getDatabase()->bindValue(':date_dep', $date_dep);
        $this->getDatabase()->bindValue(':date_arr', $date_arr);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            $row = $this->getDatabase()->fetch_assoc();
        }
        return isset($row['autobuch']) && $row['autobuch'] == 1;
    }

    public function isRateExist($rate)
    {
        $sql = "SELECT dbo.INET_GET_RATE_CODE(:rate)";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':rate', $rate);
        $this->getDatabase()->execute();
        $row = $this->getDatabase()->fetch_all();

        return count($row) > 0;
    } // isRateExist()

    public function RateCalcAufruf($rate_pk, $group, $start_time, $end_time, $station_dep_code, $station_arr_code, $km = 0)
    {
        $customer = !empty($_SESSION['DATA']['customer']) ? unserialize($_SESSION['DATA']['customer']) : null;
        if (is_a($customer, 'Customer')) {
            $customer_id = $customer->id;
        } else {
            $customer_id = 0;
        }

        $rate = null;
        $rates = array();
        $rech_pos_pk = '';
        try {
            $sql = "update next_pks set PK=PK+1 where Tablename='RATEN_LAUF'";
            $this->getDatabase()->exec($sql);
            $sql = "Select PK from next_Pks where Tablename='RATEN_LAUF'";
            $this->getDatabase()->query($sql);
            $row = $this->getDatabase()->fetch_assoc();
            $rech_pos_pk = (int) $row['PK'];
        } catch (PDOException $ex) {
            return null;
        }

        $rech_pos_pk = 'TMPRL' . $rech_pos_pk . '_1';

        $sql = "DELETE FROM RECH_POS WHERE PKRECHNUNG= :rech_pos_pk";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':rech_pos_pk', $rech_pos_pk);
        $result = $this->getDatabase()->execute();

        $sql = "SELECT Rate FROM RECH_RATEN WHERE PK = " . (int) $rate_pk;
        $this->getDatabase()->query($sql);
        $row = $this->getDatabase()->fetch_assoc();
        $rate_name = !empty($row['Rate']) ? $row['Rate'] : '';

        $sql = "EXEC RATE_CALC_AUFRUF
				@von = :start_time,
				@bis = :end_time,
				@ARate = :rate_name,
				@AGroup = :group,
				@AREPK = :rech_pos_pk,
				@KOST1 = '',
				@Kost2 = '',
				@MV = 0,
				@kmleistung = :km,
				@AKunde = :customer,
				@ARabattVorgabe =0,
				@AUFNDAT = :time,
				@Vermietstation1 = :station,
				@Zielstation1 =:station2";

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':start_time', $start_time);
        $this->getDatabase()->bindValue(':rech_pos_pk', $rech_pos_pk);
        $this->getDatabase()->bindValue(':rate_name', $rate_name);
        $this->getDatabase()->bindValue(':end_time', $end_time);
        $this->getDatabase()->bindValue(':time', date('d.m.Y H:i:s'));
        $this->getDatabase()->bindValue(':group', $group);
        $this->getDatabase()->bindValue(':customer', $customer_id);
        $this->getDatabase()->bindValue(':station', $station_dep_code);
        $this->getDatabase()->bindValue(':station2', $station_arr_code);
        $this->getDatabase()->bindValue(':km', $km);

        $this->getDatabase()->execute();

        $sql = "SELECT *,dbo.RECH_GET_BRUTTO_FROM_NETTO(kmZus,Steuerschluessel,von) AS kmbrutto from RECH_POS WHERE PKRECHNUNG = :rech_pos_pk";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':rech_pos_pk', $rech_pos_pk);
        $this->getDatabase()->execute();

        $_rates = array();
        while ($rate = $this->getDatabase()->fetch_obj()) {
            $_rates[] = $rate;
        }
        foreach ($_rates as $rate) {
            $rate->gesamtpreisbrutto = $rate->Brutto * $rate->Anzahl;
            $rate->is_angebot = true;

            if ($rate->Einzelpreis == null) {
                $rate = null;
            }
            $rate->days = $this->getDays($rech_pos_pk);
            $rates[] = $rate;
        }

        return $rates;
    } // RateCalcAufruf()

    public function getDays($tmprechpk)
    {
        $sql = "SELECT dbo.RECH_GET_BERTAGE(:tmprechpk)";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':tmprechpk', $tmprechpk);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            $row = $this->getDatabase()->fetch_array();
            if (!empty($row[0]) && $row[0] >= 1) {
                $days = round($row[0]);
            } elseif (!empty($row[0]) && $row[0] < 1) {
                $days = ceil($row[0]); // AK 27.02.2017: immer aufrunden, wenn der Wert kleiner als 1 ist, da sonst beim Einzelpreis teilweise 0 Euro raus kommt
            } else {
                $days = '';
            }
        }
        return $days;
    } //getDays

    public function getCategories()
    {
        $categories = array();
        $sql = 'SELECT * FROM RECH_RATEN_KATEGORIEN ORDER BY Kategorie';

        $this->getDatabase()->prepare($sql);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $tmp = $this->getDatabase()->fetch_all();

            foreach ($tmp as $cat) {
                $categories[$cat['PK']] = $cat;
            }
            return $categories;
        } else {
            return false;
        }
    }
} // Class DBRate
