<?php

class CX9Logging
{
    private $log_path = "";
    private $db_error_file;
    private $email_error_file;
    private $debug_file;
    private $log_file;

    public $debug = false;

    public function __construct()
    {
        if (@date_default_timezone_get() == 'UTC') {
            date_default_timezone_set(defined('DEFAULT_TIMEZONE') ? DEFAULT_TIMEZONE : "Europe/Berlin");
        }

        if (defined('LOG_PATH')) {
            $this->log_path = LOG_PATH;
        }

        $this->log_path = $this->log_path != '' ? rtrim($this->log_path, '\/') . DIRECTORY_SEPARATOR : '';
        $this->db_error_file = "db_error_" . date("Ym") . ".log";
        $this->email_error_file = "email_error_" . date("Ym") . ".log";
        $this->debug_file = "debug_" . date("Ymd") . ".log";
        $this->log_file = "log_" . date("Ymd") . ".log";
    }

    public function __call($name, $arguments)
    {
        $functions = array('log',
            'db_error_log',
            'email_error_log',
            'debug_log',
        );
        if (in_array($name, $functions) && !empty($arguments)) {
            $msg = $arguments[0];
            $full_log_path = $this->log_path;
            switch ($name) {
                case 'db_error_log':
                    $full_log_path .= $this->db_error_file;
                    break;

                case 'email_error_log':
                    $full_log_path .= $this->email_error_file;
                    break;

                case 'debug_log':
                    $full_log_path .= $this->debug_file;
                    break;

                case 'log':
                default:
                    $full_log_path .= $this->log_file;
            }

            if ($this->is_logginble($full_log_path)) {
                if (!is_string($msg)) {
                    $msg = print_r($msg, true);
                }

                return file_put_contents($full_log_path, date("d-m-Y H:i:s") . ": " . $msg . "\n", FILE_APPEND);
            }
        }

        return false;
    }

    /**
     *
     * @param string $log_path
     */
    public function set_log_path($log_path)
    {
        $this->log_path = $log_path != '' ? rtrim($log_path, '\/') . DIRECTORY_SEPARATOR : '';
    } // set_log_path()

    /**
     * @param mixed $message
     * @param boolean $to_string
     * @param boolean $titel
     */
    public function print_log($message, $to_string = false, $titel = '')
    {
        $str = '';
        if ($this->debug) {
            $bt = debug_backtrace();
            $str .= "<p>-------------------------------  ";
            $str .= $bt[0]['file'] . "  ----- LINE " . $bt[0]['line'];
            $str .= "  -------------------------------</p>";
        }
        if ($titel != '') {
            $str .= "<p>" . $titel . "</p>";
        }

        $str .= "<pre>";
        $str .= print_r($message, true);
        $str .= "</pre>";

        if ($to_string === true) {
            $str = str_replace(array('<p>', '<pre>', '</p>', '</pre>'), array('', '', "\n", "\n"), $str);
            return $str;
        } else {
            echo $str;
        }
    } // print_log()

    public function class_debug_info()
    {
        echo "<strong>log path:</strong> " . $this->log_path . "<br>";
        echo "<strong>log path exists:</strong> " . (is_dir($this->log_path) ? 'true' : 'false') . "<br>";
        echo "<strong>log path writable:</strong> " . (is_writable($this->log_path) ? 'true' : 'false') . "<br><br>";

        echo "<strong>DB error log file:</strong> " . $this->log_path . $this->db_error_file . "<br>";
        echo "<strong>DB error log file exists:</strong> " .
            (file_exists($this->log_path . $this->db_error_file) ? 'true' : 'false') . "<br>";
        echo "<strong>DB error log file writable:</strong> " .
            (is_writable($this->log_path . $this->db_error_file) ? 'true' : 'false') . "<br><br>";

        echo "<strong>e-mail error log file:</strong> " . $this->log_path . $this->email_error_file . "<br>";
        echo "<strong>e-mail error log file exists:</strong> " .
            (file_exists($this->log_path . $this->email_error_file) ? 'true' : 'false') . "<br>";
        echo "<strong>e-mail error log file writable:</strong> " .
            (is_writable($this->log_path . $this->email_error_file) ? 'true' : 'false') . "<br><br>";

        echo "<strong>debug log file:</strong> " . $this->log_path . $this->debug_file . "<br>";
        echo "<strong>debug log file exists:</strong> " .
            (file_exists($this->log_path . $this->debug_file) ? 'true' : 'false') . "<br>";
        echo "<strong>debug log file writable:</strong> " .
            (is_writable($this->log_path . $this->debug_file) ? 'true' : 'false') . "<br><br>";

        echo "<strong>log file:</strong> " . $this->log_path . $this->log_file . "<br>";
        echo "<strong>log file exists:</strong> " .
            (file_exists($this->log_path . $this->log_file) ? 'true' : 'false') . "<br>";
        echo "<strong>log file writable:</strong> " .
            (is_writable($this->log_path . $this->log_file) ? 'true' : 'false') . "<br><br>";
    } // debug()

    private function is_logginble($full_log_path)
    {
        return is_writable($this->log_path) &&
            (!file_exists($full_log_path) || is_writable($full_log_path));
    } // is_logginble()
} // Class CX9Logging
