<?php
require_once 'phpmailer/class.phpmailer.php';
require_once 'phpmailer/class.smtp.php';

class CX9Mailer extends PHPMailer
{

    private $to_adresses = '';

    private $cc_adresses = '';

    private $bcc_adresses = '';

    private $attachments = '';

    /**
     * Adds a "To" address.
     *
     * @param string $address
     * @param string $name
     * @return void
     */
    public function AddAddress($address, $name = '')
    {
        parent::AddAddress($address, $name);

        $this->to_adresses .= ($this->to_adresses === '') ? $address : ';' . $address;
    }

    /**
     * Gets all "To" adresses.
     *
     * @return string
     */
    public function getAddresses()
    {
        return $this->to_adresses;
    }

    /**
     * Clears all recipients assigned in the TO array.
     * Returns void.
     *
     * @return void
     */
    public function ClearAddresses()
    {
        parent::ClearAddresses();

        $this->to_adresses = '';
    }

    /**
     * Adds a "Cc" address.
     * Note: this function works
     * with the SMTP mailer on win32, not with the "mail"
     * mailer.
     *
     * @param string $address
     * @param string $name
     * @return void
     */
    public function AddCC($address, $name = '')
    {
        parent::AddCC($address, $name);

        $this->cc_adresses .= ($this->cc_adresses === '') ? $address : ';' . $address;
    }

    /**
     * Gets all "Cc" adresses.
     *
     * @return string
     */
    public function getCCAddresses()
    {
        return $this->cc_adresses;
    }

    /**
     * Clears all recipients assigned in the CC array.
     * Returns void.
     *
     * @return void
     */
    public function ClearCCs()
    {
        parent::ClearCCs();

        $this->cc_adresses = '';
    }

    /**
     * Adds a "Bcc" address.
     * Note: this function works
     * with the SMTP mailer on win32, not with the "mail"
     * mailer.
     *
     * @param string $address
     * @param string $name
     * @return void
     */
    public function AddBCC($address, $name = '')
    {
        parent::AddBCC($address, $name);

        $this->bcc_adresses .= ($this->bcc_adresses === '') ? $address : ';' . $address;
    }

    /**
     * Gets all "Bcc" adresses.
     *
     * @return string
     */
    public function getBCCAddresses()
    {
        return $this->bcc_adresses;
    }

    /**
     * Clears all recipients assigned in the BCC array.
     * Returns void.
     *
     * @return void
     */
    public function ClearBCCs()
    {
        parent::ClearBCCs();

        $this->bcc_adresses = '';
    }

    /**
     * Clears all recipients assigned in the TO, CC and BCC
     * array.
     * Returns void.
     *
     * @return void
     */
    public function ClearAllRecipients()
    {
        parent::ClearAllRecipients();

        $this->to_adresses = '';
        $this->cc_adresses = '';
        $this->bcc_adresses = '';
    }

    /**
     * Adds an attachment from a path on the filesystem.
     * Returns false if the file could not be found
     * or accessed.
     *
     * @param string $path
     *            Path to the attachment.
     * @param string $name
     *            Overrides the attachment name.
     * @param string $encoding
     *            File encoding (see $Encoding).
     * @param string $type
     *            File extension (MIME) type.
     * @return bool
     */
    public function AddAttachment($path, $name = '', $encoding = 'base64', $type = 'application/octet-stream', $disposition = 'attachment')
    {
        $succes = parent::AddAttachment($path, $name, $encoding, $type, $disposition);

        if ($succes) {
            $this->attachments .= ($this->attachments === '') ? $name : ';' . $name;
        }

        return $succes;
    }

    /**
     * Adds a string or binary attachment (non-filesystem) to the list.
     * This method can be used to attach ascii or binary data,
     * such as a BLOB record from a database.
     *
     * @param string $string
     *            String attachment data.
     * @param string $filename
     *            Name of the attachment.
     * @param string $encoding
     *            File encoding (see $Encoding).
     * @param string $type
     *            File extension (MIME) type.
     * @return void
     */
    public function addStringAttachment($string, $filename, $encoding = 'base64', $type = '', $disposition = 'attachment')
    {
        parent::addStringAttachment($string, $filename, $encoding, $type, $disposition);

        $this->attachments .= ($this->attachments === '') ? $filename : ';' . $filename;
    }

    /**
     * Gets all attachments.
     *
     * @return string
     */
    public function getAttachments()
    {
        return $this->attachments;
    }

    /**
     * Clears all previously set filesystem, string, and binary
     * attachments.
     * Returns void.
     *
     * @return void
     */
    public function ClearAttachments()
    {
        parent::ClearAttachments();

        $this->attachments = '';
    }
} // Class CX9Mailer
