<?php
class Cx9Smarty extends Smarty
{

    /**
     * The name of the directory where templates are located.
     *
     * @var string
     */
    public $core_template_dir = 'templates';

    /**
     * fetch the template info. Gets timestamp, and source
     * if get_source is true
     *
     * sets $source_content to the source of the template, and
     * $resource_timestamp to its time stamp
     * @param string $resource_name
     * @param string $source_content
     * @param integer $resource_timestamp
     * @param boolean $get_source
     * @param boolean $quiet
     * @return boolean
     */

    public function _fetch_resource_info(&$params)
    {
        if (!isset($params['get_source'])) {
            $params['get_source'] = true;
        }
        if (!isset($params['quiet'])) {
            $params['quiet'] = false;
        }

        $_return = false;
        $params_temp = array('resource_name' => $params['resource_name']);
        if (isset($params['resource_base_path'])) {
            $params_temp['resource_base_path'] = $params['resource_base_path'];
        } else {
            $params_temp['resource_base_path'] = $this->template_dir;
        }

        if ($this->_parse_resource_name($params_temp)) {
            $_resource_type = $params_temp['resource_type'];
            $_resource_name = $params_temp['resource_name'];
            switch ($_resource_type) {
                case 'file':
                    if ($params['get_source']) {
                        $params['source_content'] = $this->_read_file($_resource_name);
                    }
                    $params['resource_timestamp'] = filemtime($_resource_name);
                    $_return = is_file($_resource_name) && is_readable($_resource_name);
                    break;

                default:
                    // call resource functions to fetch the template source and timestamp

                    if ($params['get_source']) {
                        $_source_return = isset($this->_plugins['resource'][$_resource_type]) &&
                        call_user_func_array(
                            $this->_plugins['resource'][$_resource_type][0][0],
                            array($_resource_name, &$params['source_content'], &$this)
                        );
                    } else {
                        $_source_return = true;
                    }

                    $_timestamp_return = isset($this->_plugins['resource'][$_resource_type]) &&
                    call_user_func_array(
                        $this->_plugins['resource'][$_resource_type][0][1],
                        array($_resource_name, &$params['resource_timestamp'], &$this)
                    );

                    $_return = $_source_return && $_timestamp_return;
                    break;
            }
        }

        if (!$_return) {
            // see if we can get a template with the default template handler
            if (!empty($this->default_template_handler_func)) {
                if (!is_callable($this->default_template_handler_func)) {
                    $this->trigger_error("default template handler function \"$this->default_template_handler_func\" doesn't exist.");
                } else {
                    $_return = call_user_func_array(
                        $this->default_template_handler_func,
                        array($params_temp['resource_type'], $params_temp['resource_name'], &$params['source_content'], &$params['resource_timestamp'], &$this)
                    );
                }
            }
        }

        if (!$_return) {
            if (!$params['quiet']) {
                $this->trigger_error('unable to read resource: "' . $params['resource_name'] . '"');
            }
        } elseif ($_return && $this->security) {
            require_once SMARTY_CORE_DIR . 'core.is_secure.php';
            if (!smarty_core_is_secure($params_temp, $this)) {
                if (!$params['quiet']) {
                    $this->trigger_error('(secure mode) accessing "' . $params['resource_name'] . '" is not allowed');
                }

                $params['source_content'] = null;
                $params['resource_timestamp'] = null;
                return false;
            }
        }
        return $_return;
    }

    /**
     * parse out the type and name from the resource
     *
     * @param string $resource_base_path
     * @param string $resource_name
     * @param string $resource_type
     * @param string $resource_name
     * @return boolean
     */

    public function _parse_resource_name(&$params)
    {

        // split tpl_path by the first colon
        $_resource_name_parts = explode(':', $params['resource_name'], 2);

        if (count($_resource_name_parts) == 1) {
            // no resource type given
            $params['resource_type'] = $this->default_resource_type;
            $params['resource_name'] = $_resource_name_parts[0];
        } else {
            if (strlen($_resource_name_parts[0]) == 1) {
                // 1 char is not resource type, but part of filepath
                $params['resource_type'] = $this->default_resource_type;
                $params['resource_name'] = $params['resource_name'];
            } else {
                $params['resource_type'] = $_resource_name_parts[0];
                $params['resource_name'] = $_resource_name_parts[1];
            }
        }

        if ($params['resource_type'] == 'file') {
            if (!preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $params['resource_name'])) {
                // relative pathname to $params['resource_base_path']
                // use the first directory where the file is found
                foreach ((array) $params['resource_base_path'] as $_curr_path) {
                    $_fullpath = $_curr_path . DIRECTORY_SEPARATOR . $params['resource_name'];
                    if (file_exists($_fullpath) && is_file($_fullpath)) {
                        $params['resource_name'] = $_fullpath;
                        return true;
                    }

                    $_fullpath = $this->core_template_dir . $params['resource_name'];
                    if (file_exists($_fullpath) && is_file($_fullpath)) {
                        $params['resource_name'] = $_fullpath;
                        return true;
                    }

                    // didn't find the file, try include_path
                    $params_temp = array('file_path' => $_fullpath);
                    require_once SMARTY_CORE_DIR . 'core.get_include_path.php';
                    if (smarty_core_get_include_path($params_temp, $this)) {
                        $params['resource_name'] = $params_temp['new_file_path'];
                        return true;
                    }
                }
                return false;
            } else {
                /* absolute path */
                return file_exists($params['resource_name']);
            }
        } elseif (empty($this->_plugins['resource'][$params['resource_type']])) {
            $params_temp = array('type' => $params['resource_type']);
            require_once SMARTY_CORE_DIR . 'core.load_resource_plugin.php';
            smarty_core_load_resource_plugin($params_temp, $this);
        }

        return true;
    }
}
