<?php
class ControllerLogin extends ControllerBase
{
    public function index()
    {
        $module = new Module();

        if (!defined('FORCE_CUSTOMER_LOGIN') || FORCE_CUSTOMER_LOGIN == 0) {
            header('HTTP/1.0 403 Forbidden');
            exit;
        }

        $afterLoginStep = 'rental';
        if (defined('FORCE_CUSTOMER_LOGIN_STEP_AFTER_LOGIN') || FORCE_CUSTOMER_LOGIN_STEP_AFTER_LOGIN != '') {
            $afterLoginStep = FORCE_CUSTOMER_LOGIN_STEP_AFTER_LOGIN;
        }

        if (isset($_SESSION['DATA']['login']['isLoggedIn'])) {
            header('location: ' . SITEURL . $afterLoginStep);
            exit;
        }

        $translate = $this->loadTranslations(array('LOGIN', 'Benutzer', 'NAVI', 'CONFIRM', 'MESSAGE', 'NEW_PASSWORD', 'LOGIN_BAR', 'STEP_MENU'));

        $_SESSION['NAVI_MAX_STEP'] = 0.5;
        $_SESSION['NAVI_STEP'] = 0.5;
        $dbText = new DBText();
        $station = Station::hasStationCode(Session::getStartStation());
        $password_reset_text = $dbText->getTextByStation($station->code, 'PASSWORD_RESET_TEXT');
        $password_reset_text->mem = isset($password_reset_text->mem) ? $password_reset_text->mem : 'PASSWORD_RESET_TEXT';
        $this->template->assign('PASSWORD_RESET_TEXT', $password_reset_text->mem);
        $this->template->assign('NAVI_MAX_STEP', 0.5);
        $this->template->assign('NAVI_STEP', 0.5);
        $this->template->assign('TRANSLATE', $translate);
        $this->template->assign('TITLE', $translate['LOGIN']['KUNDENLOGIN']);
        $this->template->assign('FORCE_CUSTOMER_LOGIN', defined('FORCE_CUSTOMER_LOGIN') && FORCE_CUSTOMER_LOGIN == 1 ? true : false);

        $this->template->display('login_index.html');
    }

    public function createPassword()
    {
        $token = !empty($this->registry->args[0]) ? $this->registry->args[0] : '';

        $dbCustomer = new DBCustomer();
        $dbUserInfo = $dbCustomer->getInfoByToken($token);

        $_SESSION['NAVI_MAX_STEP'] = 1;
        $js_array = array('System.js' /*'Login.js'*/);

        $translate = $this->loadTranslations(array('Token', 'Benutzer', 'NAVI', 'CONFIRM', 'MESSAGE', 'NEW_PASSWORD', 'LOGIN_BAR', 'STEP_MENU', 'LOGIN'));

        if ($dbUserInfo === false) {
            goError(array($translate['Token']['INVALID_TOKEN']));
        }

        $this->template->assign('TITLE', $translate['Benutzer']['LOGIN_ACTIVE_TITLE']);
        $this->template->assign('TOKEN', $token);
        $this->template->assign('TRANSLATE', $translate);
        $this->template->assign('MESSAGES', json_encode($translate['MESSAGE']));
        $this->template->assign('NAVI_MAX_STEP', $_SESSION['NAVI_MAX_STEP']);
        $this->template->assign('JS_FILES', $js_array);
        $this->template->assign('NAVI_STEP', 1);
        $this->template->assign('LOGIN', 0);
        $this->template->assign('NEW_PASSWORD', 1);

        $this->template->display('global_template.html');
    } // createPassword()

    public function activatePassword()
    {
        $token = !empty($_SESSION['POST']['token']) ? $_SESSION['POST']['token'] : ((!empty($_POST['token'])) ? $_POST['token'] : '');
        $pw1 = !empty($_SESSION['POST']['field-create-pw']) ? $_SESSION['POST']['field-create-pw'] : ((!empty($_POST['newPassword'])) ? $_POST['newPassword'] : '');
        $pw2 = !empty($_SESSION['POST']['field-create-pw-repeat']) ? $_SESSION['POST']['field-create-pw-repeat'] : ((!empty($_POST['newPasswordRepeat'])) ? $_POST['newPasswordRepeat'] : '');
        $_SESSION['POST'] = array();
        unset($_SESSION['POST']);
        $dbCustomer = new DBCustomer();
        $dbUserInfo = $dbCustomer->getInfoByToken($token);
        $dbTranslate = new DBTranslate();
        $translate = $this->loadTranslations(array('Token', 'Benutzer', 'NAVI', 'CONFIRM', 'MESSAGE', 'GLOBAL', 'LOGIN_BAR', 'STEP_MENU', 'LOGIN'));

        if ($dbUserInfo === false) {
            goError(array($translate['TOKEN']['INVALID_TOKEN']));
        }

        if (strlen($pw1) < 8) {
            goError(array($translate['MESSAGE']['ERROR_PASSWORD_LENGTH']));
        }
        if ($pw1 != $pw2) {
            goError(array($translate['MESSAGE']['ERROR_PASSWORD_COMPARE']));
        }

        $user = new Customer($dbUserInfo->NUMMER);

        if (!$user->is_tmp && $user->is_inet_active) {
            goError(array($translate['BENUTZER']['ERROR_USER_ALREADY_ACTIVE']));
        } else {
            if (!$user->is_tmp && !$user->is_inet_active) {
                // nur reset password
                $dbCustomer->inetAccessActivate($user, $pw1);
                $user->tmp_num = $user->id;
                $new_id = $user->id;
            } else {
                $new_id = $dbCustomer->loginActivate($user, $pw1);
            }

            if ($new_id > 0) {
                $dbCustomer->logChanges($new_id);
                $dbCustomer->tokenDeactivate($token);
                $this->template->assign('TITLE', $translate['Benutzer']['LOGIN_ACTIVE_TITLE']);
                $this->template->assign('TRANSLATE', $translate);
                $this->template->assign('NAVI_MAX_STEP', $_SESSION['NAVI_MAX_STEP']);
                $this->template->assign('TRANSLATE', $translate);

                $this->template->assign('NAVI_STEP', 1);
                $this->template->assign('ACTIVATE_PASSWORD', 1);
                $this->template->assign('NEW_PASSWORD', 0);
                $this->template->assign('JS_FILES', array('System.js'));

                $this->template->display('global_template.html');
                exit;
            } else {
                goError(array($translate['BENUTZER']['ERROR_USER_ACTIVATE']));
            }
        }
    } // activatePassword
} // Class ControllerLogin
